/* ***** BEGIN LICENSE BLOCK *****

 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * ***** END LICENSE BLOCK ***** */

function BmhRDF() {
    // namespaces
    const NC_NS = "http://home.netscape.com/NC-rdf#";
    const RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
    this.stylesNS = "http://bookmarkshome.mozdev.org/rdf/styles/";
    // types
    const nsIRDFResource = Components.interfaces.nsIRDFResource;
    const nsIRDFLiteral =  Components.interfaces.nsIRDFLiteral;
    // services
    this.RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
	.getService( Components.interfaces.nsIRDFService );
    this.RDFC = Components.classes["@mozilla.org/rdf/container;1"]
	.createInstance( Components.interfaces.nsIRDFContainer );
    this.RDFCU = Components.classes["@mozilla.org/rdf/container-utils;1"]
	.getService( Components.interfaces.nsIRDFContainerUtils );
    this.ISEARCHSVC = Components.classes["@mozilla.org/rdf/datasource;1?name=internetsearch"]
	.getService( Components.interfaces.nsIInternetSearchService );
    // datasources
    this.BMDS  = this.RDF.GetDataSource( "rdf:bookmarks" );
    this.ISDS = this.RDF.GetDataSource( "rdf:internetsearch" );
    // arcs 
    this.NameArc = this.RDF.GetResource( NC_NS + "Name" );
    this.URLArc =  this.RDF.GetResource( NC_NS + "URL" );
    this.IconArc =  this.RDF.GetResource( NC_NS + "Icon" );
    this.DescriptionArc = this.RDF.GetResource( NC_NS + "Description" );
    this.typeArc = this.RDF.GetResource( RDF_NS + "type" );
    this.childArc = this.RDF.GetResource( NC_NS + "child" );
    // style arcs
    this.nameArc = this.RDF.GetResource( this.stylesNS + "name" );
    this.sheetArc = this.RDF.GetResource( this.stylesNS + "sheet" );
    // roots
    this.bmRoot = this.RDF.GetResource( "NC:BookmarksRoot" );
    this.rootEngine = this.RDF.GetResource( "NC:SearchEngineRoot" );
}


var bmhLib = {

getLocaleString : function( strName, formattingValues ) {
		      var str = "???";
		      try {
			  var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
			      .getService( Components.interfaces.nsIStringBundleService );
			  var strBundle = sbs.createBundle( "chrome://bookmarkshome/locale/bmh.properties" );
			  if ( arguments.length >= 2 ) {
			      str = strBundle.formatStringFromName( strName, formattingValues, formattingValues.length );
			  } else {
			      str = strBundle.GetStringFromName( strName );
			  }
		      } 
		      catch( e ) {}
		      return str;
		  },

getNrOfPrefs: function() {
		  /*
		  var count = {value:0}, childArray;
		  var branch = bmhLib.getBranch( "bookmarkshome." );
		  branch.getChildList( "", count );
		  return count.value;
		  */
		  // hard code, because error if unused prefs
		  return 11;
	      },

getPrefType: function( name, bStr ) {
		 var branchString = ( arguments.length == 2 ? bStr : "bookmarkshome." );
		 var branch = bmhLib.getBranch( branchString ); 
		 return branch.getPrefType( name );
	     },


getPref: function( name, bStr ) {
	     var branchString = ( arguments.length == 2 ? bStr : "bookmarkshome." );
	     var branch = bmhLib.getBranch( branchString ); 
	     var type = branch.getPrefType( name );
	     if( type == branch.PREF_STRING ) {
		 return branch.getCharPref( name );
	     }
	     else if( type == branch.PREF_INT ) {
		 return branch.getIntPref( name ); 
	     }
	     else if( type == branch.PREF_BOOL ) {
		 return branch.getBoolPref( name );
	     }
	 },

openURL: function( url ) {
		var browser = bmhLib.getMainWindow().getBrowser().selectedBrowser;
		browser.loadURI( url  );
	    },

getBranch: function( bStr ) {
	       return Components.classes["@mozilla.org/preferences-service;1"]
		   .getService( Components.interfaces.nsIPrefService )
		   .getBranch( bStr );
	   },


setPref: function( name, type, value ) {
	     try {
		 var branch = bmhLib.getBranch( "bookmarkshome." );
		 if( type == branch.PREF_STRING ) {
		     branch.setCharPref( name, value );
		 }
		 else if( type == branch.PREF_INT ) {
		     branch.setIntPref( name, value ); 
		 }
		 else if( type == branch.PREF_BOOL ) {
		     branch.setBoolPref( name, value );
		 }
	     }
	     catch( e ) {}
	 },

getProperty: function ( aInput, aArc, DS ) {
		 try {
		     var node;
		     node = DS.GetTarget( aInput, aArc, true );
		     if( node instanceof Components.interfaces.nsIRDFResource ) {
			 return node.Value;
		     }
		     if( node instanceof Components.interfaces.nsIRDFLiteral ) {
			 return node.Value;
		     }
		 }
		 catch( e ) {}
		 return "";
	     },

getStylesFile: function() {
		   // get path to profile folder to retrieve/save rdf styles file
		   var  profileDirString = Components.classes["@mozilla.org/file/directory_service;1"] 
		       .getService( Components.interfaces.nsIProperties )
		       .get( "ProfD", Components.interfaces.nsIFile )
		       .QueryInterface( Components.interfaces.nsILocalFile )
		       .path; 
		   // create rdf styles file instance and initialize
		   var stylesFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
		   if( stylesFile instanceof Components.interfaces.nsILocalFile ) {
		       stylesFile.initWithPath( profileDirString );
		       stylesFile.append( "styles.rdf" );
		   }
		   return stylesFile;
	       },

getStylesFileURL: function( stylesFile ) {
		      return Components.classes["@mozilla.org/network/io-service;1"]
			  .getService( Components.interfaces.nsIIOService )
			  .newFileURI( stylesFile )
			  .spec;
		  },

selectEngine: function( dfltOption ) {
		  try {
		      var win = this.getMainWindow().getBrowser().selectedBrowser.contentWindow;
		      var doc = win.document;
		      var searchForm = doc.getElementById( "searchForm" );
		      var engineSelect = doc.getElementById( "engineSelect" );
		      var options = engineSelect.getElementsByTagName( "option" );
		      var engineImg = doc.getElementById( "engineImg" );
		      var displayValue = ""; 
		      if(( ! dfltOption ) || ( dfltOption.style.display == "none" )) {
			  // select the first not hidden engine
			  for( var i = 0; i < options.length; i++ ) {
			      if( options[i].style.display != "none" ) {
				  options[i].selected = true;
				  searchForm.style.display = displayValue;
				  engineImg.src = options[i].value.split( "|" )[1];
				  return;
			      }
			  }
			  // all engines are hidden
			  displayValue = "none";
		      }
		      else if( dfltOption ) {
			  // dfltOption is not hidden
			  dfltOption.selected = true;
			  engineImg.src = dfltOption.value.split( "|" )[1];
		      }
		      searchForm.style.display = displayValue;
		  }
		  catch( e ) {}
	      },

testProps: function( object, propChain ) {
	       try {
		   var rv = object;
		   for( var i = 0; i < propChain.length; i++ ) {
		       rv = rv[propChain[i]];
		   }
		   return rv;
	       }
	       catch( e ) {}
	       return null;
	   },

getStylesDS: function( rdf ) {
		 // get styles file
		 var stylesFile = bmhLib.getStylesFile();
		 // get url of styles file
		 var stylesFileURL = bmhLib.getStylesFileURL( stylesFile );
		 // on first run after installation, rdf styles file does not exist. 
		 // cannot be saved to chrome url, but must be changeable
		 //var rdf.RDF = bmhLib.getRDF();
		 if( ! stylesFile.exists()) {
		     rdf.RDF.GetDataSourceBlocking( "chrome://bookmarkshome/content/styles.rdf" )
			 .QueryInterface( Components.interfaces.nsIRDFRemoteDataSource )
			 .FlushTo( stylesFileURL );		      
		 }
		 // get rdf styles datasource 
		 return rdf.RDF.GetDataSourceBlocking( stylesFileURL ); 
	     },

getIconDisplay: function( type, displayIndex ) {
		    if(( type == "Folder" && displayIndex == 0 ) ||
			    ( type == "Livemark" && displayIndex < 2 )) {
			return "inline";
		    }
		    return "none";
		},

formatSheet: function( sheet ) {
		 sheet = sheet.replace( /(\*\/)\s*/g, "$1\n" );
		 sheet = sheet.replace( /([\{;])\s*/g, "$1\n" );
		 sheet = sheet.replace( /(\})\s*/g, "$1\n\n" );
		 sheet = sheet.replace( /(.*;)/g, "    $1" );
		 return sheet;
	     },

binarySearch: function( target, array ) {
		  var found = false;
		  var first = 0, last = array.length - 1, temp, mid;
		  while (( !found ) && ( first <= last )) {
		      temp = first + last;
		      // integer division
		      mid = ( temp - ( temp % 2 )) / 2;
		      if( target == array[mid] ) {
			  found = true;
		      }
		      else if( target < array[mid] ) {
			  last = mid - 1;
		      }
		      else {
			  first = mid + 1;
		      }
		  }
		  return( found ? mid : -1 ); 
	      }, 

getArrayFromPref: function( pref ) {
		      var str = bmhLib.getPref( pref );
		      var arr = ( str == "" ? [] : str.split( "|" ));
		      for( var i = 0; i < arr.length; i++ ) {
			  arr[i] = decodeURI( arr[i] );
		      }
		      return arr;
		  },

getPrefFromArray: function( arr ) {
		      var pref = "";
		      for ( var i = 0; i < arr.length; i++ ) {
			  pref += ( "|" + encodeURI( arr[i] ));
		      }
		      if( pref.length > 0 ) {
			  // remove first bar
			  pref = pref.substr( 1 );
		      }
		      return pref;
		  },

isExcludedFolder: function( folderString, excludeIndex, inOrExcludedFolders ) {
		      var index = bmhLib.binarySearch( folderString, inOrExcludedFolders ); 
		      var isInOrExcluded = ( index >= 0 );
		      var exclude = ( bmhLib.getPref( "excludeIndex" ) == 0 ); 
		      return ( exclude == isInOrExcluded );
		  },

getMainWindow: function() {
		   try {
		       var win = top;
		       while( win.opener ) {
			   win = win.opener.top;
		       }
		       return win.QueryInterface( Components.interfaces.nsIInterfaceRequestor )
			   .getInterface( Components.interfaces.nsIWebNavigation )
			   .QueryInterface( Components.interfaces.nsIDocShellTreeItem )
			   .rootTreeItem
			   .QueryInterface( Components.interfaces.nsIInterfaceRequestor )
			   .getInterface( Components.interfaces.nsIDOMWindow );
		   }
		   catch( e ) {
		       return null;
		   }
	       },

openFolder: function( listId ) {
		var links = document.getElementById( listId ).getElementsByTagName( "a" );
		var i;
		var inTabs = ( bmhLib.getPref( "openFolderIn" ) == 0 );
		if( inTabs ) {
		    // thanks to Neil Deakin
		    var browser = bmhLib.getMainWindow().getBrowser();
		    for( i = 0; i < links.length; i++ ) { 
			browser.addTab( links[i].getAttribute( "href" ));
		    }
		}
		else {
		    // open in new windows
		    for( i = 0; i < links.length; i++ ) {
			window.open( links[i].getAttribute( "href" ));
		    }
		}
	    }
}


